# Ein Spielstein wird mit dem Wert 0 und einem x und einem y initialisiert.
class GamePiece:
    def __init__(self, row, column):
        self.row = row
        self.column = column
        self.value = 0
